
if { $argc != 1 } {
    puts "The script requires the project name to be set as an argument."	
} else {

    start_gui
    
    set proj_name [lindex $argv 0]
    set script_path [ file dirname [ file normalize [ info script ] ] ]
    set vivado_dir_path ../vivado
    set ip_repo_dir_path ../ix_repo
    set hdl_dir_path ../hdl
    set constraint_dir_path ../constraints

    create_project $proj_name $vivado_dir_path/$proj_name -part xc7z030fbg676-3

    set_property target_language VHDL [current_project]
    
    # Work-around suggested by Xilinx to avoid the following inconsequential critical warning
    # [Memdata 28-165] The reference name: top_i_ip_m_blk_mem_gen_0 was not found in a previous reference definition. Either the bmm file or the bmm_info_* properties are malformed, therefore BRAM INIT strings can not be populated.
    # Source: https://support.xilinx.com/s/article/59639?language=en_US
    set_param memdata.disableAnnotatedBMM 1
    set_param memdata.disableBRAMInitStrings 1
    set_param memdata.ignoreEmbeddedBMMProperties 1

    set_property  ip_repo_paths  $ip_repo_dir_path [current_project]
    update_ip_catalog
    create_bd_design "top"
    create_bd_cell -type ip -vlnv imperix.ch:ix:IXIP:3.10 IXIP_0
    set_property name IXIP [get_bd_cells IXIP_0]

    add_files -fileset constrs_1 -norecurse $constraint_dir_path/sandbox_pins_3.10rev1.xdc

    make_bd_intf_pins_external  [get_bd_intf_pins IXIP/FIXED_IO]
    make_bd_intf_pins_external  [get_bd_intf_pins IXIP/DDR]
    make_bd_intf_pins_external  [get_bd_intf_pins IXIP/BBOX]
    make_bd_intf_pins_external  [get_bd_intf_pins IXIP/USR]

    set_property name FIXED_IO [get_bd_intf_ports FIXED_IO_0]
    set_property name DDR [get_bd_intf_ports DDR_0]
    set_property name BBOX [get_bd_intf_ports BBOX_0]
    set_property name USR [get_bd_intf_ports USR_0]

    make_bd_pins_external  [get_bd_pins IXIP/private_out]
    make_bd_pins_external  [get_bd_pins IXIP/pwm]
    make_bd_pins_external  [get_bd_pins IXIP/gpo]
    make_bd_pins_external  [get_bd_pins IXIP/private_in]
    make_bd_pins_external  [get_bd_pins IXIP/flt]
    make_bd_pins_external  [get_bd_pins IXIP/gpi]

    set_property name private_out [get_bd_ports private_out_0]
    set_property name pwm [get_bd_ports pwm_0]
    set_property name gpo [get_bd_ports gpo_0]
    set_property name private_in [get_bd_ports private_in_0]
    set_property name flt [get_bd_ports flt_0]
    set_property name gpi [get_bd_ports gpi_0]

    create_bd_cell -type ip -vlnv xilinx.com:ip:xlconstant:1.1 xlconstant_0
    set_property name USER_ID [get_bd_cells xlconstant_0]
    set_property -dict [list CONFIG.CONST_WIDTH {16}] [get_bd_cells USER_ID]
    set_property -dict [list CONFIG.CONST_VAL {1}] [get_bd_cells USER_ID]
    connect_bd_net [get_bd_pins USER_ID/dout] [get_bd_pins IXIP/user_fw_id]

    make_wrapper -files [get_files $vivado_dir_path/$proj_name/$proj_name.srcs/sources_1/bd/top/top.bd] -top
    import_files -force -norecurse $vivado_dir_path/$proj_name/$proj_name.gen/sources_1/bd/top/hdl/top_wrapper.vhd

    add_files -norecurse $hdl_dir_path/AXIS_interface.vhd
    create_bd_cell -type module -reference ix_axis_interface ix_axis_interface_0
    set_property name ix_axis_interface [get_bd_cells ix_axis_interface_0]
    connect_bd_intf_net [get_bd_intf_pins IXIP/SBIO_BUS] [get_bd_intf_pins ix_axis_interface/SBIO_BUS]
    connect_bd_intf_net [get_bd_intf_pins IXIP/ADC] [get_bd_intf_pins ix_axis_interface/ADC]
    connect_bd_net [get_bd_pins IXIP/adc_done_pulse] [get_bd_pins ix_axis_interface/adc_done_pulse_in]
    connect_bd_net [get_bd_pins IXIP/data_valid_pulse] [get_bd_pins ix_axis_interface/data_valid_pulse_in]
    connect_bd_net [get_bd_pins IXIP/reading] [get_bd_pins ix_axis_interface/data_reading_in]
    connect_bd_net [get_bd_pins IXIP/clk_250_mhz] [get_bd_pins ix_axis_interface/clk]
    connect_bd_net [get_bd_pins IXIP/sync_pulse] [get_bd_pins ix_axis_interface/sync_pulse_in]

    # Configuration of IXIP
    set_property -dict [list CONFIG.SB_GT_SFP_0 {true} CONFIG.SB_GT_SFP_1 {true}] [get_bd_cells IXIP]

    # Instantiation of Aurora 64B/66B 1
    create_bd_cell -type ip -vlnv xilinx.com:ip:aurora_64b66b:12.0 aurora_64b66b_0
    set_property -dict [list CONFIG.C_LINE_RATE {5} CONFIG.C_REFCLK_FREQUENCY {250.000}] [get_bd_cells aurora_64b66b_0]

    # Instantiation of Aurora 64B/66B 2
    create_bd_cell -type ip -vlnv xilinx.com:ip:aurora_64b66b:12.0 aurora_64b66b_1
    set_property -dict [list CONFIG.C_LINE_RATE {5} CONFIG.C_REFCLK_FREQUENCY {250.000}] [get_bd_cells aurora_64b66b_1]

    # Instantiation of Clock Wizard 1
    create_bd_cell -type ip -vlnv xilinx.com:ip:clk_wiz:6.0 clk_wiz_0
    set_property -dict [list CONFIG.PRIM_SOURCE {Global_buffer} CONFIG.PRIM_IN_FREQ {156.250} CONFIG.CLKOUT2_USED {true} CONFIG.CLKOUT1_REQUESTED_OUT_FREQ {78.125} CONFIG.CLKOUT2_REQUESTED_OUT_FREQ {156.250} CONFIG.USE_LOCKED {false} CONFIG.USE_RESET {false} CONFIG.CLKIN1_JITTER_PS {64.0} CONFIG.MMCM_DIVCLK_DIVIDE {1} CONFIG.MMCM_CLKFBOUT_MULT_F {6.000} CONFIG.MMCM_CLKIN1_PERIOD {6.400} CONFIG.MMCM_CLKIN2_PERIOD {10.0} CONFIG.MMCM_CLKOUT0_DIVIDE_F {12.000} CONFIG.MMCM_CLKOUT1_DIVIDE {6} CONFIG.NUM_OUT_CLKS {2} CONFIG.CLKOUT1_JITTER {126.299} CONFIG.CLKOUT1_PHASE_ERROR {96.563} CONFIG.CLKOUT2_JITTER {110.242} CONFIG.CLKOUT2_PHASE_ERROR {96.563}] [get_bd_cells clk_wiz_0]

    # Instantiation of Clock Wizard 2
    create_bd_cell -type ip -vlnv xilinx.com:ip:clk_wiz:6.0 clk_wiz_1
    set_property -dict [list CONFIG.PRIM_SOURCE {Global_buffer} CONFIG.PRIM_IN_FREQ {156.250} CONFIG.CLKOUT2_USED {true} CONFIG.CLKOUT1_REQUESTED_OUT_FREQ {78.125} CONFIG.CLKOUT2_REQUESTED_OUT_FREQ {156.250} CONFIG.USE_LOCKED {false} CONFIG.USE_RESET {false} CONFIG.CLKIN1_JITTER_PS {64.0} CONFIG.MMCM_DIVCLK_DIVIDE {1} CONFIG.MMCM_CLKFBOUT_MULT_F {6.000} CONFIG.MMCM_CLKIN1_PERIOD {6.400} CONFIG.MMCM_CLKIN2_PERIOD {10.0} CONFIG.MMCM_CLKOUT0_DIVIDE_F {12.000} CONFIG.MMCM_CLKOUT1_DIVIDE {6} CONFIG.NUM_OUT_CLKS {2} CONFIG.CLKOUT1_JITTER {126.299} CONFIG.CLKOUT1_PHASE_ERROR {96.563} CONFIG.CLKOUT2_JITTER {110.242} CONFIG.CLKOUT2_PHASE_ERROR {96.563}] [get_bd_cells clk_wiz_1]

    # Instantiation of Processor System Reset
    create_bd_cell -type ip -vlnv xilinx.com:ip:proc_sys_reset:5.0 proc_sys_reset_0

    # Instantiation of constants
    create_bd_cell -type ip -vlnv xilinx.com:ip:xlconstant:1.1 xlconstant_0
    create_bd_cell -type ip -vlnv xilinx.com:ip:xlconstant:1.1 xlconstant_1
    create_bd_cell -type ip -vlnv xilinx.com:ip:xlconstant:1.1 xlconstant_2
    set_property -dict [list CONFIG.CONST_WIDTH {32} CONFIG.CONST_VAL {0}] [get_bd_cells xlconstant_2]

    # Instantiation of FIFOs (clock domain crossing)
    create_bd_cell -type ip -vlnv xilinx.com:ip:axis_data_fifo:2.0 axis_data_fifo_0
    set_property -dict [list CONFIG.TDATA_NUM_BYTES.VALUE_SRC USER CONFIG.TDATA_NUM_BYTES {8} CONFIG.FIFO_DEPTH {16} CONFIG.IS_ACLK_ASYNC {1}] [get_bd_cells axis_data_fifo_0]
    create_bd_cell -type ip -vlnv xilinx.com:ip:axis_data_fifo:2.0 axis_data_fifo_1
    set_property -dict [list CONFIG.TDATA_NUM_BYTES.VALUE_SRC USER CONFIG.TDATA_NUM_BYTES {8} CONFIG.FIFO_DEPTH {16} CONFIG.IS_ACLK_ASYNC {1}] [get_bd_cells axis_data_fifo_1]
    create_bd_cell -type ip -vlnv xilinx.com:ip:axis_data_fifo:2.0 axis_data_fifo_2
    set_property -dict [list CONFIG.TDATA_NUM_BYTES.VALUE_SRC USER CONFIG.TDATA_NUM_BYTES {8} CONFIG.FIFO_DEPTH {16} CONFIG.IS_ACLK_ASYNC {1}] [get_bd_cells axis_data_fifo_2]
    create_bd_cell -type ip -vlnv xilinx.com:ip:axis_data_fifo:2.0 axis_data_fifo_3
    set_property -dict [list CONFIG.TDATA_NUM_BYTES.VALUE_SRC USER CONFIG.TDATA_NUM_BYTES {8} CONFIG.FIFO_DEPTH {16} CONFIG.IS_ACLK_ASYNC {1}] [get_bd_cells axis_data_fifo_3]

    # Instantiation of concat blocks
    create_bd_cell -type ip -vlnv xilinx.com:ip:xlconcat:2.1 xlconcat_0
    set_property -dict [list CONFIG.IN1_WIDTH.VALUE_SRC USER CONFIG.IN0_WIDTH.VALUE_SRC USER] [get_bd_cells xlconcat_0]
    set_property -dict [list CONFIG.IN0_WIDTH {32} CONFIG.IN1_WIDTH {32}] [get_bd_cells xlconcat_0]
    create_bd_cell -type ip -vlnv xilinx.com:ip:xlconcat:2.1 xlconcat_1
    set_property -dict [list CONFIG.IN1_WIDTH.VALUE_SRC USER CONFIG.IN0_WIDTH.VALUE_SRC USER] [get_bd_cells xlconcat_1]
    set_property -dict [list CONFIG.IN0_WIDTH {32} CONFIG.IN1_WIDTH {32}] [get_bd_cells xlconcat_1]

    # Instantiation of slice blocks
    create_bd_cell -type ip -vlnv xilinx.com:ip:xlslice:1.0 xlslice_0
    set_property -dict [list CONFIG.DIN_WIDTH {64} CONFIG.DIN_TO {0} CONFIG.DIN_FROM {31} CONFIG.DOUT_WIDTH {32}] [get_bd_cells xlslice_0]
    create_bd_cell -type ip -vlnv xilinx.com:ip:xlslice:1.0 xlslice_1
    set_property -dict [list CONFIG.DIN_WIDTH {64} CONFIG.DIN_TO {0} CONFIG.DIN_FROM {31} CONFIG.DOUT_WIDTH {32}] [get_bd_cells xlslice_1]

    # Connections
    connect_bd_net [get_bd_pins IXIP/rxn_0] [get_bd_pins aurora_64b66b_0/rxn]
    connect_bd_net [get_bd_pins IXIP/rxp_0] [get_bd_pins aurora_64b66b_0/rxp]
    connect_bd_net [get_bd_pins IXIP/txn_0] [get_bd_pins aurora_64b66b_0/txn]
    connect_bd_net [get_bd_pins IXIP/txp_0] [get_bd_pins aurora_64b66b_0/txp]
    connect_bd_net [get_bd_pins IXIP/rxn_1] [get_bd_pins aurora_64b66b_1/rxn]
    connect_bd_net [get_bd_pins IXIP/rxp_1] [get_bd_pins aurora_64b66b_1/rxp]
    connect_bd_net [get_bd_pins IXIP/txn_1] [get_bd_pins aurora_64b66b_1/txn]
    connect_bd_net [get_bd_pins IXIP/txp_1] [get_bd_pins aurora_64b66b_1/txp]

    connect_bd_net [get_bd_pins IXIP/GT_gt_refclk1] [get_bd_pins aurora_64b66b_1/refclk1_in]
    connect_bd_net [get_bd_pins IXIP/GT_gt_refclk1] [get_bd_pins aurora_64b66b_0/refclk1_in]
    connect_bd_net [get_bd_pins IXIP/GT_gt_qpllclk_quad1] [get_bd_pins aurora_64b66b_1/gt_qpllclk_quad1_in]
    connect_bd_net [get_bd_pins IXIP/GT_gt_qpllclk_quad1] [get_bd_pins aurora_64b66b_0/gt_qpllclk_quad1_in]
    connect_bd_net [get_bd_pins IXIP/GT_gt_qpllrefclk_quad1] [get_bd_pins aurora_64b66b_1/gt_qpllrefclk_quad1_in]
    connect_bd_net [get_bd_pins IXIP/GT_gt_qpllrefclk_quad1] [get_bd_pins aurora_64b66b_0/gt_qpllrefclk_quad1_in]

    connect_bd_net [get_bd_pins IXIP/clk_50_mhz] [get_bd_pins aurora_64b66b_0/init_clk]
    connect_bd_net [get_bd_pins IXIP/clk_50_mhz] [get_bd_pins aurora_64b66b_0/drp_clk_in]
    connect_bd_net [get_bd_pins IXIP/clk_50_mhz] [get_bd_pins aurora_64b66b_1/init_clk]
    connect_bd_net [get_bd_pins IXIP/clk_50_mhz] [get_bd_pins aurora_64b66b_1/drp_clk_in]
    connect_bd_net [get_bd_pins IXIP/clk_50_mhz] [get_bd_pins proc_sys_reset_0/slowest_sync_clk]

    connect_bd_net [get_bd_pins xlconstant_0/dout] [get_bd_pins proc_sys_reset_0/dcm_locked]
    connect_bd_net [get_bd_pins xlconstant_0/dout] [get_bd_pins proc_sys_reset_0/aux_reset_in]
    connect_bd_net [get_bd_pins xlconstant_0/dout] [get_bd_pins proc_sys_reset_0/ext_reset_in]

    connect_bd_net [get_bd_pins proc_sys_reset_0/bus_struct_reset] [get_bd_pins aurora_64b66b_0/pma_init]
    connect_bd_net [get_bd_pins proc_sys_reset_0/bus_struct_reset] [get_bd_pins aurora_64b66b_1/pma_init]
    connect_bd_net [get_bd_pins proc_sys_reset_0/peripheral_reset] [get_bd_pins aurora_64b66b_0/reset_pb]
    connect_bd_net [get_bd_pins proc_sys_reset_0/peripheral_reset] [get_bd_pins aurora_64b66b_1/reset_pb]

    connect_bd_net [get_bd_pins clk_wiz_0/clk_in1] [get_bd_pins aurora_64b66b_0/tx_out_clk]
    connect_bd_net [get_bd_pins clk_wiz_0/clk_out1] [get_bd_pins aurora_64b66b_0/user_clk]
    connect_bd_net [get_bd_pins clk_wiz_0/clk_out2] [get_bd_pins aurora_64b66b_0/sync_clk]
    connect_bd_net [get_bd_pins clk_wiz_1/clk_in1] [get_bd_pins aurora_64b66b_1/tx_out_clk]
    connect_bd_net [get_bd_pins clk_wiz_1/clk_out1] [get_bd_pins aurora_64b66b_1/user_clk]
    connect_bd_net [get_bd_pins clk_wiz_1/clk_out2] [get_bd_pins aurora_64b66b_1/sync_clk]

    #connect_bd_intf_net [get_bd_intf_pins axis_data_fifo_0/S_AXIS] [get_bd_intf_pins ix_axis_interface/M_AXIS_CPU2FPGA_00]
    connect_bd_intf_net [get_bd_intf_pins axis_data_fifo_0/M_AXIS] [get_bd_intf_pins aurora_64b66b_0/USER_DATA_S_AXIS_TX]
    #connect_bd_intf_net [get_bd_intf_pins axis_data_fifo_1/S_AXIS] [get_bd_intf_pins ix_axis_interface/M_AXIS_CPU2FPGA_01]
    connect_bd_intf_net [get_bd_intf_pins axis_data_fifo_1/M_AXIS] [get_bd_intf_pins aurora_64b66b_1/USER_DATA_S_AXIS_TX]
    connect_bd_intf_net [get_bd_intf_pins axis_data_fifo_2/S_AXIS] [get_bd_intf_pins aurora_64b66b_0/USER_DATA_M_AXIS_RX]
    #connect_bd_intf_net [get_bd_intf_pins axis_data_fifo_2/M_AXIS] [get_bd_intf_pins ix_axis_interface/S_AXIS_FPGA2CPU_00]
    connect_bd_intf_net [get_bd_intf_pins axis_data_fifo_3/S_AXIS] [get_bd_intf_pins aurora_64b66b_1/USER_DATA_M_AXIS_RX]
    #connect_bd_intf_net [get_bd_intf_pins axis_data_fifo_3/M_AXIS] [get_bd_intf_pins ix_axis_interface/S_AXIS_FPGA2CPU_01]

    connect_bd_net [get_bd_pins xlconstant_1/dout] [get_bd_pins axis_data_fifo_0/s_axis_aresetn]
    connect_bd_net [get_bd_pins xlconstant_1/dout] [get_bd_pins axis_data_fifo_1/s_axis_aresetn]
    connect_bd_net [get_bd_pins xlconstant_1/dout] [get_bd_pins axis_data_fifo_2/s_axis_aresetn]
    connect_bd_net [get_bd_pins xlconstant_1/dout] [get_bd_pins axis_data_fifo_3/s_axis_aresetn]

    connect_bd_net [get_bd_pins IXIP/clk_250_mhz] [get_bd_pins axis_data_fifo_0/s_axis_aclk]
    connect_bd_net [get_bd_pins IXIP/clk_250_mhz] [get_bd_pins axis_data_fifo_1/s_axis_aclk]
    connect_bd_net [get_bd_pins IXIP/clk_250_mhz] [get_bd_pins axis_data_fifo_2/m_axis_aclk]
    connect_bd_net [get_bd_pins IXIP/clk_250_mhz] [get_bd_pins axis_data_fifo_3/m_axis_aclk]

    connect_bd_net [get_bd_pins clk_wiz_0/clk_out1] [get_bd_pins axis_data_fifo_0/m_axis_aclk]
    connect_bd_net [get_bd_pins clk_wiz_1/clk_out1] [get_bd_pins axis_data_fifo_1/m_axis_aclk]
    connect_bd_net [get_bd_pins clk_wiz_0/clk_out1] [get_bd_pins axis_data_fifo_2/s_axis_aclk]
    connect_bd_net [get_bd_pins clk_wiz_1/clk_out1] [get_bd_pins axis_data_fifo_3/s_axis_aclk]

    connect_bd_net [get_bd_pins xlconcat_0/In1] [get_bd_pins xlconstant_2/dout]
    connect_bd_net [get_bd_pins xlconcat_1/In1] [get_bd_pins xlconstant_2/dout]
    connect_bd_net [get_bd_pins xlconcat_0/In0] [get_bd_pins ix_axis_interface/M_AXIS_CPU2FPGA_00_tdata]
    connect_bd_net [get_bd_pins xlconcat_1/In0] [get_bd_pins ix_axis_interface/M_AXIS_CPU2FPGA_01_tdata]

    connect_bd_net [get_bd_pins axis_data_fifo_0/s_axis_tdata] [get_bd_pins xlconcat_0/dout]
    connect_bd_net [get_bd_pins axis_data_fifo_1/s_axis_tdata] [get_bd_pins xlconcat_1/dout]
    connect_bd_net [get_bd_pins axis_data_fifo_0/s_axis_tvalid] [get_bd_pins ix_axis_interface/M_AXIS_CPU2FPGA_00_tvalid]
    connect_bd_net [get_bd_pins axis_data_fifo_0/s_axis_tready] [get_bd_pins ix_axis_interface/M_AXIS_CPU2FPGA_00_tready]
    connect_bd_net [get_bd_pins axis_data_fifo_1/s_axis_tvalid] [get_bd_pins ix_axis_interface/M_AXIS_CPU2FPGA_01_tvalid]
    connect_bd_net [get_bd_pins axis_data_fifo_1/s_axis_tready] [get_bd_pins ix_axis_interface/M_AXIS_CPU2FPGA_01_tready]

    connect_bd_net [get_bd_pins xlslice_0/Dout] [get_bd_pins ix_axis_interface/S_AXIS_FPGA2CPU_00_tdata]
    connect_bd_net [get_bd_pins xlslice_1/Dout] [get_bd_pins ix_axis_interface/S_AXIS_FPGA2CPU_01_tdata]
    connect_bd_net [get_bd_pins axis_data_fifo_2/m_axis_tdata] [get_bd_pins xlslice_0/Din]
    connect_bd_net [get_bd_pins axis_data_fifo_2/m_axis_tvalid] [get_bd_pins ix_axis_interface/S_AXIS_FPGA2CPU_00_tvalid]
    connect_bd_net [get_bd_pins axis_data_fifo_2/m_axis_tready] [get_bd_pins ix_axis_interface/S_AXIS_FPGA2CPU_00_tready]
    connect_bd_net [get_bd_pins axis_data_fifo_3/m_axis_tdata] [get_bd_pins xlslice_1/Din]
    connect_bd_net [get_bd_pins axis_data_fifo_3/m_axis_tvalid] [get_bd_pins ix_axis_interface/S_AXIS_FPGA2CPU_01_tvalid]
    connect_bd_net [get_bd_pins axis_data_fifo_3/m_axis_tready] [get_bd_pins ix_axis_interface/S_AXIS_FPGA2CPU_01_tready]

    regenerate_bd_layout

    save_bd_design
}